eakcpv <- function(s1, r1, u1, s0, r0, u0, conf, imp, cycl)
{
    library(mice)
            
    if (u1 == 0 & u0 == 0)
     {
       stop("Frequencies u1 and u0 are zero. These data do not correspond to a problem with missing data. Introduce new values \n")
     }
      
    if (s1 < 0 | r1 < 0 | u1 < 0 | s0 < 0 | r0 < 0 | u0 < 0) 
     {
       stop("Frequencies cannot be negative. Introduces new values \n")
     }
      
    if (abs(s1 - trunc (s1)) > 0  | abs(r1 - trunc (r1)) > 0  | abs(u1 - trunc (u1)) > 0  | abs(s0 - trunc (s0)) > 0  | abs(r0 - trunc (r0)) > 0  | abs(u0 - trunc (u0)) > 0)
     {
       stop("Frequencies cannot have decimals. Introduces new values \n")
     } 
      
    if (conf >= 1 | conf <= 0) 
     {
       stop("Confidence level should take a value between 0 and 1. Introduces a new value \n")
     }
      
    if (imp <= 1 | abs(imp - trunc(imp)) > 0) 
     {
       stop("Number of complete data sets is incorrect. Introduces a new integer positive value (> 1) \n")
     }
      
    if (cycl <= 1 | abs(cycl - trunc(cycl)) > 0) 
     {
       stop("Number of cycles is incorrect. Introduces a new integer positive value (> 1) \n")
     }          
      
    fre <- c(s1, r1, u1, s0, r0, u0)
      
    z <- qnorm(1 - (1 - conf) / 2, 0, 1)
    
    niter <- imp
      
    n1 = s1 + r1 + u1
    n0 = s0 + r0 + u0
    n =  n0 + n1
       
    # Function to combine the results (Rubin's rule)
    combi <- function(v1, v2)
     {
       meanv1 <- mean(v1)
       meanv2 <- mean(v2)
       b <- (1 / (imp - 1)) * sum((v1 - meanv1)^2)
       vari <- meanv2 + ((imp + 1) / imp) * b
       dof <- (imp - 1) *((imp * vari) / ((imp + 1) * b))^2
       Stud <- qt(1 - (1 - conf) / 2, dof)
       Lconfi <- meanv1 - Stud * sqrt(vari)
       Uconfi <- meanv1 + Stud * sqrt(vari)
       result <- list(est = meanv1, Var = vari, Lci = Lconfi, Uci = Uconfi)
       result 
     }        
         
    # Multiple Imputation: MICE method
    impdata <- matrix(0, 1, 4)
    
    vkappa0m <- vector("integer", imp)
    vkappa1m <- vector("integer", imp)
                
    vk1m <- vector("integer", imp)
    vk2m <- vector("integer", imp)      
    
    varcsink1m <- vector("integer", imp)
    varcsink2m <- vector("integer", imp)

    vVark1m <- vector("integer", imp)
    vVark2m <- vector("integer", imp) 
    
    vVararcsink1m <- vector("integer", imp)
    vVararcsink2m <- vector("integer", imp)      
          
    GS <- rep(c(1, 0, NA), 2)
    bdt <- c(rep(1, 3),rep(0, 3))
    mat1 <- cbind(GS, bdt)
    mat2 <- cbind(mat1, fre)
    w <- 1
    dat1 <- matrix("integer", n, 2)
    
    #If u1 or u2 is equal 0
    if (u1 == 0 | u0 ==0)  
       {
        if (u1 == 0)
           {
            mat2 <- rbind(mat2[1 : 2,], mat2[4 : 6,])
           }
    
        for (i in 1 : 5)
           {
             for (j in 1:mat2[i, 3])
              {
                dat1[w,] <- mat2[i, 1:2 ]
                w <- w + 1
              }
           }
        }
    else
        {           
         for (i in 1: 6)
           {
             for (j in 1 : mat2[i, 3])
              {
                dat1[w,] <- mat2[i, 1 : 2]
                w <- w + 1
              }
           }
         }  
        
    dat2 <- as.data.frame(dat1)
    
    set.seed(1231)
    
    k <- niter
    
    while (k >= 1)
      {     
        imput <- mice(dat2, method = "logreg", m = 1, maxit = cycl)
    
        impdata <- table(complete(imput))
      
        xt1 <- impdata[4]
        xt0 <- impdata[2]
        yt1 <- impdata[3]
        yt0 <- impdata[1]
               
        mt1 <- xt1 + yt1
        mt0 <- xt0 + yt0
        mtt <- mt1 + mt0  
        
        Set <- xt1 / (xt1 + xt0)
        Spt <- yt0 / (yt1 + yt0)      
                
        Yt <- Set + Spt - 1
                
        if(Yt > 0) 
          {        
            x1 <- impdata[4]
            x0 <- impdata[2]
            y1 <- impdata[3]
            y0 <- impdata[1]
            
            cat(x1,x0,y1,y0,"\n")
                   
            m1 <- x1 + y1
            m0 <- x0 + y0
            mt <- m1 + m0  
            
            Sem <- x1 / (x1 + x0)
            Spm <- y0 / (y1 + y0)
            pm <- (x1 + x0) / mt       
                    
            Ym <- Sem + Spm - 1
                        
            Qm <- pm * Sem + (1 - pm) * (1 - Spm)
                
            VarSem <- Sem * (1 - Sem) / (x1 + x0) 
            VarSpm <- Spm * (1 - Spm) / (y1 + y0) 
            Varpm <- pm * (1 - pm) / mt
      
            kappa0m <- (Spm - (1 - Qm)) / Qm          
            Varkappa0m <- ((1 - Spm)^2 * Ym^2 * Varpm + pm^2 * ((1 - Spm)^2 * VarSem + Sem^2 * VarSpm)) / Qm^4
    
            kappa1m <- (Sem - Qm) / (1 - Qm)         
            Varkappa1m <- ((1 - Sem)^2 * Ym^2 * Varpm + (1 - pm)^2 * (Spm^2 * VarSem + (1 - Sem)^2 * VarSpm)) / (1 - Qm)^4
            
            if (abs(x0 - y1) > 0) 
             {  
              k1m <- (2 * (x1 * y0 - x0 * y1) / (m0 * (x1 + x0) - m1 * (y1 + y0))) * log((m1 * (y1 + y0) + m0 * (x1 + x0)) / (2 * m1 * (y1 + y0)))    
              
              arcsink1m <- asin(sqrt(k1m)) 
            
              Vark1m <- (1 /((kappa0m + kappa1m)^2 * (kappa0m - kappa1m)^2)) * 
              (((2 * kappa0m^2 * kappa1m - kappa1m * (kappa0m + kappa1m) * k1m) / kappa0m)^2 * 
              ((1 - Spm)^2 * Ym^2 * Varpm + pm^2 * ((1 - Spm)^2 * VarSem + Sem^2 * VarSpm)) /  Qm^4 + 
              (kappa0m * ((kappa0m + kappa1m) * k1m - 2 * kappa0m * kappa1m) / kappa1m)^2 * 
              ((1 - Sem)^2 * Ym^2 * Varpm + (1 - pm)^2 * (Spm^2 * VarSem + (1 - Sem)^2 * VarSpm)) / (1 - Qm)^4 +     
              2 * ((2 * kappa0m^2 * kappa1m - kappa1m * (kappa0m + kappa1m) * k1m) / kappa0m) * 
              (kappa0m * ((kappa0m + kappa1m) * k1m - 2 * kappa0m * kappa1m) / kappa1m) * 
              ((pm * (1 - pm) * ((1 - Sem) * Sem * VarSpm + (1 - Spm) * Spm * VarSem) - (1 - Sem) * (1 - Spm) * Ym^2 * Varpm) / (Qm^2 * (1 - Qm)^2)))
              
              Vararcsink1m <- 1 / (4 * k1m * (1 - k1m)) *  Vark1m
                  
              k2m <- (2 * (x1 * y0 - x0 * y1) / (m0 * (x1 + x0) - m1 * (y1 + y0))) * log((2 * m0 * (x1 + x0)) / (m1 * (y1 + y0) + m0 * (x1 + x0)))
              
              arcsink2m <- asin(sqrt(k2m))
              
              Vark2m <- (1 /((kappa0m + kappa1m)^2 * (kappa0m - kappa1m)^2)) * 
              ((kappa1m * (2 * kappa0m * kappa1m - (kappa0m + kappa1m) * k2m)/ kappa0m)^2 * 
              ((1 - Spm)^2 * Ym^2 * Varpm + pm^2 * ((1 - Spm)^2 * VarSem + Sem^2 * VarSpm)) /  Qm^4 + 
              ((kappa0m * (kappa0m + kappa1m) * k2m - 2 * kappa0m * kappa1m^2) / kappa1m)^2 * 
              ((1 - Sem)^2 * Ym^2 * Varpm + (1 - pm)^2 * (Spm^2 * VarSem + (1 - Sem)^2 * VarSpm)) / (1 - Qm)^4 +     
              2 * (kappa1m * (2 * kappa0m * kappa1m - (kappa0m + kappa1m) * k2m)/ kappa0m) * 
              ((kappa0m * (kappa0m + kappa1m) * k2m - 2 * kappa0m * kappa1m^2) / kappa1m) * 
              ((pm * (1 - pm) * ((1 - Sem) * Sem * VarSpm + (1 - Spm) * Spm * VarSem) - (1 - Sem) * (1 - Spm) * Ym^2 * Varpm) / (Qm^2 * (1 - Qm)^2)))
              
              Vararcsink2m <- 1 / (4 * k2m * (1 - k2m)) *  Vark2m         
             }      
            else
             {  
              k1m <- Ym 
              
              k2m <- Ym
                
              Vark1m <- VarSem + VarSpm
              
              Vark2m <- Vark1m
              
              Vararcsink1m <- 1 / (4 * k1m * (1 - k1m)) *  Vark1m
              
              Vararcsink2m <- Vararcsink1m                       
             }
          
          
         vkappa0m[k] <- kappa0m
         vkappa1m[k] <- kappa1m
         
         vk1m[k] <- k1m
         vVark1m[k] <- Vark1m
         
         vk2m[k] <- k2m
         vVark2m[k] <- Vark2m
                      
         varcsink1m[k] <- asin(sqrt(k1m))
         vVararcsink1m[k] <- Vararcsink1m
         
         varcsink2m[k] <- asin(sqrt(k2m))
         vVararcsink2m[k] <- Vararcsink2m 
         
         impdata <- matrix(0, 1, 4)      
   
         k <- k - 1
         
         }
      }
      
      cat(vk1m,"\n")
      cat(vk2m,"\n")
    
    # Rubin's rule: Wald CI  
    kappa0.mean <- mean(vkappa0m)
    kappa1.mean <- mean(vkappa1m)
    
    k1.combi <- combi(vk1m, vVark1m)
    LmiceWaldk1 <- k1.combi$Lci
    UmiceWaldk1 <- k1.combi$Uci
        
    arcink1.combi <- combi(varcsink1m, vVararcsink1m)    
    Lmicearcsink1 <- arcink1.combi$Lci
    Umicearcsink1 <- arcink1.combi$Uci
    
    k2.combi <- combi(vk2m, vVark2m)
    LmiceWaldk2 <- k2.combi$Lci
    UmiceWaldk2 <- k2.combi$Uci
    
    arcink2.combi <- combi(varcsink2m, vVararcsink2m)    
    Lmicearcsink2 <- arcink2.combi$Lci
    Umicearcsink2 <- arcink2.combi$Uci
    
    f1 <- function(x) (k1.combi$est - (kappa0.mean * kappa1.mean / (kappa0.mean * x + kappa1.mean * (1 - x))))
    c1 <- uniroot(f1, c(0, 1))[[1]] 
    
    f2 <- function(x) (k2.combi$est - (kappa0.mean * kappa1.mean / (kappa0.mean * x + kappa1.mean * (1 - x))))
    c2 <- uniroot(f2, c(0, 1))[[1]]   
         
    # OUTPUT
    sink("results_eakcpv.txt", split=TRUE)
    cat("\n")
    cat("\n")      
    cat("\n")
    cat("Observed frequencies are: \n")
    cat("                Positive Test          Negative Test \n")
    cat("Verified \n")      
    cat("  Diseased      ", s1,"                     ", s0," \n")      
    cat("  Non Diseased  ", r1,"                     ",  r0," \n")      
    cat("Non Verified    ", u1,"                     ", u0," \n")      
    cat("\n")
    cat("\n")
    cat("\n")
    cat("     MULTIPLE IMPUTATION: MICE METHOD \n")
    cat("------------------------------------------ \n")
    cat("\n")
    cat("\n")
    cat("Estimated K(0) is ",kappa0.mean, "\n")
    cat("Estimated K(1) is ",kappa1.mean, "\n")
    cat("\n")
    cat("\n")  
    cat("     AVERAGE KAPPA COEFFICIENT FOR  L' > L  (0 < c < 0.5) \n")
    cat("\n")     
    cat("Estimated average kappa coefficient (K1) is ",k1.combi$est," and its standard error is ", sqrt(k1.combi$Var), "\n")       
    cat("\n")
    cat(100 * conf,"% Wald CI for K1: ", LmiceWaldk1," ; ", UmiceWaldk1, "\n")          
    cat("\n")
    cat(100 * conf,"% Arcsine CI for K1: ", sin(Lmicearcsink1)^2," ; ", sin(Umicearcsink1)^2, "\n")          
    cat("\n")
    cat("Estimated relative loss between the false positives and the false negatives is: ", c1, "\n")
    cat("\n")
    cat("Loss associated with the false positives (L') is ", (1 - c1) / c1, " times greater than the loss associated with the false negatives (L)", "\n")          
    cat("\n")
    cat("\n")
    cat("     AVERAGE KAPPA COEFFICIENT FOR  L > L'  (0.5 < c < 1) \n")
    cat("\n")     
    cat("Estimated average kappa coefficient (K2) is ",k2.combi$est," and its standard error is ", sqrt(k2.combi$Var), "\n")       
    cat("\n")
    cat(100 * conf,"% Wald CI for K2: ", LmiceWaldk2," ; ", UmiceWaldk2, "\n")          
    cat("\n")
    cat(100 * conf,"% Arcsine CI for K2: ", sin(Lmicearcsink2)^2," ; ", sin(Umicearcsink2)^2, "\n")
    cat("\n")
    cat("\n")
    cat("Estimated relative loss between the false positives and the false negatives is: ", c2, "\n")
    cat("\n")
    cat("Loss associated with the false negatives (L) is ", c2 / (1 - c2), " times greater than the loss associated with the false positives (L')", "\n")          
    cat("\n") 
    sink()    
}
